#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for _cache_symbols() private method"""

from pymarketcap import Pymarketcap
from pymarketcap.consts import EXCEPTIONAL_COIN_SLUGS
pym = Pymarketcap()

def test_types():
    res = pym._cache_symbols_ids()
    assert isinstance(res, tuple)
    for i, _property in enumerate(res):
        assert isinstance(_property, dict)
        for key, value in _property.items():
            if i == 0:
                assert isinstance(value, str)
            else:
                assert isinstance(value, int)

def test_consistence():
    res = pym.correspondences
    for key, value in res.items():
        assert " " not in value
        for ch in key:
            try:
                assert ch.isupper() or ch.isnumeric() or ch in ["-", "$", "@"]
            except AssertionError:
                assert ch.islower()
        for ch in value:
            assert ch.islower() or ch.isnumeric() or ch in ["-"]
    for original, correct in EXCEPTIONAL_COIN_SLUGS.items():
        assert res[original] == correct



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Pymarketcap installation precompiler."""

import os
from precompiler.builder import Builder

def run_builder(source):
    print("Precompiling pymarketcap source code...")
    builder = Builder(source)

    build_funcs = [
        "ticker_badges",
        "unskip_tests"
    ]

    try:
        for func in build_funcs:
            builder.build(func)
    except Exception as e:
        print("Error precompiling pymarketcap source code.")
        return False

    print("Pymarketcap source code precompiled successfully.")
    return True

def run_unbuilder():
    print("Restoring pymarketcap source code...")
    builder = Builder()
    try:
        response = builder.unbuild()
    except Exception as e:
        print("Error restoring pymarketcap source code.")
        return False
    print("Pymarketcap source code restored successfully.")
    return response



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from time import sleep
from random import choice
from urllib.request import urlopen
from tqdm import tqdm

import pytest

from pymarketcap import Pymarketcap
pym = Pymarketcap()

coin = choice(pym.coins)
exchange = choice(pym.exchange_slugs)

endpoints = [
    ("_cache_symbols_ids",     "https://s2.coinmarketcap.com/generated/search/quick_search.json"),
    ("_exchange_names_slugs",  "https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"),
    ("ticker",            ["https://api.coinmarketcap.com/v1/ticker/",
                           "https://api.coinmarketcap.com/v1/ticker/%s" % coin]),
    ("markets",            "https://coinmarketcap.com/currencies/%s/" % coin),
    ("ranks",              "https://coinmarketcap.com/gainers-losers/"),
    ("historical",         "https://coinmarketcap.com/currencies/%s/historical-data/" % coin),
    ("recently",           "https://coinmarketcap.com/new/"),
    ("exchange",           "https://coinmarketcap.com/exchanges/%s/" % exchange),
    ("exchanges",          "https://coinmarketcap.com/exchanges/volume/24-hour/all/"),
    ("tokens",             "https://coinmarketcap.com/tokens/views/all/"),
    ("graphs.currency",    "https://graphs2.coinmarketcap.com/currencies/%s/" % coin),
    ("graphs.global_cap", ["https://graphs2.coinmarketcap.com/global/marketcap-total/",
                           "https://graphs2.coinmarketcap.com/global/marketcap-altcoin/"]),
    ("graphs.dominance",   "https://graphs2.coinmarketcap.com/global/dominance/"),
    ("download_logo",      "https://s2.coinmarketcap.com/static/img/coins/64x64/1.png")
]

def assert_up(ep):
    req = urlopen(ep)
    assert req.getcode() != 404
    req.close()

@pytest.mark.end2end
def test_endpoints():
    for i, (method, eps) in enumerate(tqdm(endpoints)):
        if i == 0:
            tqdm.write("tests/test_endpoints.py::test_endpoints")
        if type(eps) != list:
            eps = [eps]
        for ep in eps:
            tqdm.write("%18s() --> %s" % (method, ep))
            assert_up(ep)
            sleep(.1)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Asynchronous Pymarketcap interface module."""

# Standard Python modules
import logging
from json import loads
from json.decoder import JSONDecodeError
from datetime import datetime
from asyncio import (
    ensure_future,
    Queue,
)
from asyncio import TimeoutError as AsyncioTimeoutError

# External Python dependencies
from aiohttp import ClientSession
from tqdm import tqdm

# Internal Cython modules
from pymarketcap import Pymarketcap
from pymarketcap import processer

# Internal Python modules
from pymarketcap.consts import (
    DEFAULT_TIMEOUT,
    EXCEPTIONAL_COIN_SLUGS,
    DEFAULT_FORMATTER
)

# Logging initialization
LOGGER_NAME = "/pymarketcap%s" % __file__.split("pymarketcap")[-1]
LOGGER = logging.getLogger(LOGGER_NAME)
LOGGER_HANDLER = logging.StreamHandler()
LOGGER_HANDLER.setFormatter(DEFAULT_FORMATTER)
LOGGER.addHandler(LOGGER_HANDLER)

class AsyncPymarketcap(ClientSession):
    """Asynchronous scraper for coinmarketcap.com

    Args:
        queue_size (int): Number of maximum simultanenous
           get requests performing together in methods
           involving several requests. As default ``10``.
        progress_bar (bool): Select ``True`` or ``False`` if you
            want to show a progress bar in methods that involve
            processing of several requests (requires :mod:`tqdm`
            module). As default, ``True``.
        consumers (int): Number of consumers resolving HTTP
            requests from an internal
            :class:`asyncio.Queue <~asyncio.Queue>`.
            As default, ``10``.
        timeout (int/float, optional): Limit max time
            waiting for a response. As default, ``15``.
        logger (logging.logger): As default is a logger
            with a :class:`~python.logging.StreamHandler`.
        debug (bool, optional): If ``True``, the logger
            level will be setted as :data:`~logging.DEBUG`.
            As default ``False``.
        **kwargs: arguments that corresponds to the
            :class:`aiohttp.client.ClientSession <~aiohttp.ClientSession>`
            parent class.
    """

    def __init__(self, queue_size=10, progress_bar=True,
                 consumers=10, timeout=DEFAULT_TIMEOUT,
                 logger=LOGGER, debug=False, json=None,
                 **kwargs):
        super(AsyncPymarketcap, self).__init__(**kwargs)
        self.timeout = timeout
        self.logger = logger
        self.sync = Pymarketcap()

        # Async queue
        self.queue_size = queue_size
        self.connector_limit = self.connector.limit
        self._responses = []
        self.progress_bar = progress_bar
        self.consumers = consumers

        self.graphs = type("Graphs", (), self._graphs_interface)

        if debug:
            self.logger.setLevel(logging.DEBUG)

    # PROPERTIES

    @property
    def correspondences(self):
        try:
            return self._correspondences
        except AttributeError:
            self._correspondences = self.sync._cache_symbols_ids()[0]
            return self._correspondences

    @property
    def symbols(self):
        try:
            return self._symbols
        except AttributeError:
            self._symbols = self.sync.symbols
            return self._symbols

    @property
    def coins(self):
        try:
            return self._coins
        except AttributeError:
            self._coins = self.sync.coins
            return self._coins

    @property
    def exchange_names(self):
        """Get all exchange formatted names provided by coinmarketcap."""
        try:
            return self._exchange_names
        except AttributeError:
            self._exchange_names = self.sync.exchange_names
            return self._exchange_names

    @property
    def exchange_slugs(self):
        """Get all exchange raw names provided by coinmarketcap."""
        try:
            return self._exchange_slugs
        except AttributeError:
            self._exchange_slugs = self.sync.exchange_slugs
            return self._exchange_slugs

    @property
    def total_currencies(self):
        """Get all exchange raw names provided by coinmarketcap."""
        try:
            return self._total_currencies
        except AttributeError:
            self._total_currencies = self.sync.total_currencies
            return self._total_currencies

    @property
    def total_exchanges(self):
        """Get all exchange raw names provided by coinmarketcap."""
        try:
            return self._total_exchanges
        except AttributeError:
            self._total_exchanges = self.sync.total_exchanges
            return self._total_exchanges

    # UTILS

    @property
    def _graphs_interface(self):
        return {
            "currency": self._currency,
            "every_currency": self._every_currency,
            "global_cap": self._global_cap,
            "dominance": self._dominance
        }

    def _is_symbol(self, currency):
        if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
            if currency in self.sync.__repeated_symbols:
                msg = 'The symbol "%s" has more than one correspondence ' % currency \
                    + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \
                    + "\nPossible valid slug names: %r." % self.sync.__repeated_symbols[currency]
                raise ValueError(msg)
            return True
        return False

    async def _cache_symbols(self):
        url = "https://s2.coinmarketcap.com/generated/search/quick_search.json"
        res = await self._get(url)
        symbols = {}
        for currency in loads(res):
            symbols[currency["symbol"]] = currency["slug"].replace(" ", "")
        for original, correct in EXCEPTIONAL_COIN_SLUGS.items():
            symbols[original] = correct
        return symbols

    async def _get(self, url):
        async with self.get(url, timeout=self.timeout) as response:
            return await response.text()

    async def _async_multiget(self, itr, build_url_callback,
                              num_of_consumers=None, desc=""):
        queue, dlq, responses = Queue(maxsize=self.queue_size), Queue(), []
        num_of_consumers = num_of_consumers or min(self.connector_limit, self.try_get_itr_len(itr))
        consumers = [ensure_future(
            self._consumer(main_queue=queue, dlq=dlq, responses=responses)) for _ in
                     range(num_of_consumers or self.connector_limit)]
        dlq_consumers = [ensure_future(
            self._consumer(dlq, dlq, responses)) for _ in range(num_of_consumers)]
        await self._producer(itr, build_url_callback, queue, desc=desc)
        await queue.join()
        await dlq.join()

        all_consumers = consumers
        all_consumers.extend(dlq_consumers)
        for consumer in all_consumers:
            consumer.cancel()
        return responses

    def try_get_itr_len(self, itr):
        try:
            return len(itr)
        except TypeError:
            return 1000000

    async def _producer(self, items,
                        build_url_callback,
                        queue, desc=""):
        for item in tqdm(items, desc=desc, disable=not self.progress_bar):
            await queue.put(await build_url_callback(item))

    async def _consumer(self, main_queue, dlq, responses):
        while True:
            try:
                url = await main_queue.get()
                responses.append([url, await self._get(url)])
                # Notify the queue that the item has been processed
                main_queue.task_done()
            except AsyncioTimeoutError:
                self.logger.debug("Problem with %s, Moving to DLQ" % url)
                await dlq.put(url)
                main_queue.task_done()

    # SCRAPER
    async def _base_currency_url(self, name):
        if self._is_symbol(name):
            name = self.correspondences[name]
        return "https://coinmarketcap.com/currencies/%s" % name

    async def currency(self, name, convert="USD"):
        res = await self._get(self._base_currency_url(name))
        convert = convert.lower()
        return processer.currency(res[20000:], convert)

    async def every_currency(self, currencies=None,
                             convert="USD", consumers=None):
        """Return general data from every currency in coinmarketcap
        passing a list of currencies as first parameter.
        As default returns data for all currencies.

        Args:
            currencies (list, optional): Iterator with all the
                currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            convert (str, optional): Convert prices in response
                between "USD" and BTC. As default ``"USD"``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (list): Data for all currencies.
        """
        convert = convert.lower()
        currencies = currencies if currencies else self.coins
        res = await self._async_multiget(
            currencies,
            self._base_currency_url,
            consumers if consumers else self.consumers,
            desc="Retrieving every currency data " \
                 + "for %d currencies from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            response = processer.currency(raw_res[20000:], convert)
            response["slug"] = url.split("/")[-1]
            for symbol, slug in self.correspondences.items():
                if slug == response["slug"]:
                    response["symbol"] = symbol
                    break
            yield response

    async def markets(self, name, convert="USD"):
        res = await self._get(self._base_currency_url(name))
        convert = convert.lower()
        return processer.markets(res[20000:], convert)

    async def every_markets(self, currencies=None,
                            convert="USD", consumers=None):
        """Returns markets data from every currency in coinmarketcap
        passing a list of currencies as first parameter.
        As default returns data for all currencies.

        Args:
            currencies (list, optional): Iterator with
                all the currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            convert (str, optional): Convert prices in
                response between "USD" and BTC.
                As default ``"USD"``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator): Data for all currencies.
        """
        convert = convert.lower()
        currencies = currencies if currencies else self.coins
        res = await self._async_multiget(
            currencies,
            self._base_currency_url,
            consumers if consumers else self.consumers,
            desc="Retrieving all markets " \
                 + "for %d currencies from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            response = {"markets": processer.markets(raw_res[20000:], convert),
                        "slug": url.split("/")[-1]}
            for symbol, slug in self.correspondences.items():
                if slug == response["slug"]:
                    response["symbol"] = symbol
                    break
            yield response

    async def ranks(self):
        res = await self._get("https://coinmarketcap.com/gainers-losers/")
        return processer.ranks(res)

    async def _base_historical_url(self, name):
        if self._is_symbol(name):
            name = self.correspondences[name]
        url = "https://coinmarketcap.com/currencies/%s/historical-data" % name
        _start = "%d%02d%02d" % (self.__start.year, self.__start.month, self.__start.day)
        _end = "%d%02d%02d" % (self.__end.year, self.__end.month, self.__end.day)
        url += "?start=%s&end=%s" % (_start, _end)
        return url

    async def historical(self, name,
                         start=datetime(2008, 8, 18),
                         end=datetime.now(),
                         revert=False):
        self.__start = start
        self.__end = end
        res = await self._get(self._base_historical_url(name))
        return processer.historical(res[50000:], start, end, revert)

    async def every_historical(self, currencies=None,
                               start=datetime(2008, 8, 18),
                               end=datetime.now(),
                               revert=False,
                               consumers=None):
        """Returns historical data from every currency
        in coinmarketcap passing a list of currencies
        as first parameter. As default returns data
        for all currencies.

        Args:
            currencies (list, optional): Iterator with all
                the currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            start (date, optional): Time to start scraping
                periods as ``datetime.datetime`` type.
                As default ``datetime(2008, 8, 18)``.
            end (date, optional): Time to end scraping periods
                as datetime.datetime type.
                As default ``datetime.now()``.
            revert (bool, optional): If ``False``, return first date
                first, in chronological order, otherwise returns
                reversed list of periods. As default ``False``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator):
            Historical data for all currencies.
        """
        self.__start = start
        self.__end = end
        currencies = currencies if currencies else self.coins
        res = await self._async_multiget(
            currencies,
            self._base_historical_url,
            consumers if consumers else self.consumers,
            desc="Retrieving all historical data " \
                 + "for %d currencies from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            response = {"history": processer.historical(raw_res[50000:],
                                                        start, end, revert),
                        "slug": url.split("/historical-data")[0].split("/")[-1]}
            for symbol, slug in self.correspondences.items():
                if slug == response["slug"]:
                    response["symbol"] = symbol
                    break
            yield response

    async def recently(self, convert="USD"):
        convert = convert.lower()
        url = "https://coinmarketcap.com/new/"
        res = await self._get(url)
        return list(processer.recently(res, convert))

    async def _base_exchange_url(self, name):
        return "https://coinmarketcap.com/exchanges/%s" % name

    async def exchange(self, name, convert="USD"):
        convert = convert.lower()
        res = await self._get(_base_exchange_url(name))[20000:]
        return processer.exchange(res, convert)

    async def every_exchange(self, exchanges=None, convert="USD", consumers=None):
        """Returns general data from every exchange
        in coinmarketcap passing a list of exchanges
        as first parameter. As default returns data
        for all exchanges.

        Args:
            exchanges (list, optional): Iterator with all
                the exchanges that you want to retrieve.
                As default ``None``
                (:meth:`pymarketcap.Pymarketcap.exchange_slugs`
                will be used in that case).
            convert (str, optional): Convert market_caps, prices,
                volumes and percent_changes between USD and BTC.
                As default ``"USD"``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator):
            General data from all exchanges.
        """
        convert = convert.lower()
        exchanges = exchanges if exchanges else self.exchange_slugs
        res = await self._async_multiget(
            exchanges,
            self._base_exchange_url,
            consumers if consumers else self.consumers,
            desc="Retrieving all exchange data " \
                 + "for %d exchanges from coinmarketcap" % len(exchanges)
        )
        for url, raw_res in res:
            response = processer.exchange(raw_res, convert)
            response["slug"] = url.split("/")[-1]
            yield response

    async def exchanges(self, convert="USD"):
        url = "https://coinmarketcap.com/exchanges/volume/24-hour/all/"
        convert = convert.lower()
        res = await self._get(url)[45000:]
        return processer.exchanges(res, convert)

    async def tokens(self, convert="USD"):
        url = "https://coinmarketcap.com/tokens/views/all/"
        convert = convert.lower()
        res = await self._get(url)[40000:]
        return processer.tokens(res, convert)

    # GRAPHS API
    async def _base_graphs_currency_url(self, name):
        if self._is_symbol(name):
            name = self.correspondences[name]
        return "https://graphs2.coinmarketcap.com/currencies/%s" % name

    async def _currency(self, name, start=None, end=None):
        res = await self._get(self._base_graphs_currency_url(name))
        return processer.graphs(loads(res), start, end)

    async def _every_currency(self, currencies=None,
                              start=None, end=None, consumers=None):
        """Returns graphs data from every currency in
        coinmarketcap passing a list of currencies as
        first parameter. As default returns graphs data
        for all currencies.

        Args:
            currencies (list, optional): Iterator with all
                the currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            start (datetime, optional): Time to start retrieving
                graphs data in datetime. As default ``None``.
            end (datetime, optional): Time to end retrieving
                graphs data in datetime. As default ``None``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator): Graphs data from all currencies.
        """
        currencies = currencies if currencies else self.coins
        res = await self._async_multiget(
            currencies,
            self._base_graphs_currency_url,
            consumers if consumers else self.consumers,
            desc="Retrieving all graphs data " \
                 + "for %d currencies from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            try:
                raw_res = loads(raw_res)
            except JSONDecodeError:  # Ignore 404 responses
                continue
            response = processer.graphs(raw_res, start, end)
            response["slug"] = url.split("/")[-1]
            for symbol, slug in self.correspondences.items():
                if slug == response["slug"]:
                    response["symbol"] = symbol
                    break
            yield response

    async def _global_cap(self, bitcoin=True, start=None, end=None):
        if bitcoin:
            url = "https://graphs2.coinmarketcap.com/global/marketcap-total/"
        else:
            url = "https://graphs2.coinmarketcap.com/global/marketcap-altcoin/"
        return processer.graphs(loads(self._get(url)), start, end)

    async def _dominance(self, start=None, end=None):
        url = "https://graphs2.coinmarketcap.com/global/dominance/"
        return processer.graphs(loads(self._get(url)), start, end)




"""API wraper and web scraper module."""

# Standard Python modules
from re import sub as re_sub
from re import findall as re_findall
from json import loads
from datetime import datetime
from time import time
from urllib.request import urlretrieve
from urllib.error import HTTPError

# Internal Cython modules
from pymarketcap.curl import get_to_memory
from pymarketcap import processer

# Internal Python modules
from pymarketcap.consts import (
    DEFAULT_TIMEOUT,
    EXCEPTIONAL_COIN_SLUGS,
    INVALID_COINS
)
from pymarketcap.errors import (
    CoinmarketcapHTTPError,
    CoinmarketcapHTTPError404,
    CoinmarketcapTooManyRequestsError
)

# HTTP errors mapper
http_errors_map = {
    "429": CoinmarketcapTooManyRequestsError,
    "404": CoinmarketcapHTTPError404,
}

http_error_numbers = [int(number) for number in http_errors_map.keys()]


cdef class Pymarketcap:
    """Synchronous class for retrieve data from coinmarketcap.com

    Args:
        timeout (int, optional): Set timeout value for get requests.
            As default ``20``.
        debug: (bool, optional): Show low level data in get requests.
            As default, ``False``.
    """
    cdef readonly dict _correspondences
    cdef readonly dict _ids_correspondences
    cdef readonly list _symbols
    cdef readonly list _coins
    cdef readonly int  _total_currencies
    cdef readonly int  _total_exchanges
    cdef readonly list _currencies_to_convert
    cdef readonly list _converter_cache
    cdef readonly list _exchange_names
    cdef readonly list _exchange_slugs
    cdef readonly dict __repeated_symbols

    cdef public long timeout
    cdef public object graphs
    cdef public bint debug

    def __init__(self, timeout=DEFAULT_TIMEOUT, debug=False):
        self.timeout = timeout
        self.debug = debug

        #: object: Initialization of graphs internal interface
        self.graphs = type("Graphs", (), self._graphs_interface)

    # ====================================================================

                         #######   UTILS   #######

    @property
    def _graphs_interface(self):
        return {
            "currency": self._currency,
            "global_cap": self._global_cap,
            "dominance": self._dominance
        }

    @property
    def correspondences(self):
        res = self._correspondences
        if res:
            return res
        else:
            main_cache = self._cache_symbols_ids()
            self._correspondences = main_cache[0]
            self._ids_correspondences = main_cache[1]
            return self._correspondences

    @property
    def ids_correspondences(self):
        """Get symbols with their correspondient numeric
        id (used for debug purposes)."""
        res = self._ids_correspondences
        if res:
            return res
        else:
            main_cache = self._cache_symbols_ids()
            self._correspondences = main_cache[0]
            self._ids_correspondences = main_cache[1]
            return self._ids_correspondences

    cpdef _is_symbol(self, unicode currency):
        """Internal function for check
        if a currency string may be a symbol or not.
        This function is not strict, so if you pass
        self._is_symbol("OBASDFPAFFFOUASVBF") will returns True
        but in this context we don't need to check if
        the user has introduced a valid symbol.

        Returns bint:
            1 if True, 0 if False
        """
        cdef bint response
        response = 0
        if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
            try:
                if currency in self.__repeated_symbols:
                    msg = 'The symbol "%s" has more than one correspondence ' % currency \
                        + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \
                        + "\nPossible valid slug names: %r." % self.__repeated_symbols[currency]
                    raise ValueError(msg)
            except TypeError:
                pass
            response = 1
        return response

    cpdef _quick_search(self):
        """Internal pymarketcap JSON cache file ``quick_search.json``."""
        cdef bytes url
        url = b"https://s2.coinmarketcap.com/generated/search/quick_search.json"
        return loads(self._get(url))

    cpdef _cache_symbols_ids(self):
        """Internal function for load in cache all symbols
        in coinmarketcap with their respectives currency names."""
        self.__repeated_symbols = {}
        symbols_slugs, symbols_ids = {}, {}
        for currency in self._quick_search():
            symbol = currency["symbol"]
            slug = currency["slug"].replace(" ", "")
            if symbol in symbols_slugs:    # Repeated symbols are stored internally
                if symbol in self.__repeated_symbols.keys():
                    self.__repeated_symbols[symbol].append(slug)
                else:
                    self.__repeated_symbols[symbol] = [symbols_slugs[symbol], slug]
            else:
                symbols_slugs[symbol] = slug
            symbols_ids[symbol] = currency["id"]
        for original, correct in EXCEPTIONAL_COIN_SLUGS.items():
            symbols_slugs[original] = correct
        return (symbols_slugs, symbols_ids)

    @property
    def symbols(self):
        """Symbols of currencies (in capital letters).

        Returns (list):
            All currency symbols provided by coinmarketcap.
        """
        res = self._symbols
        if res:
            return res
        else:
            self._symbols = sorted(list(self.correspondences.keys()))
            for invalid_symbol in self.__repeated_symbols:
                self._symbols.remove(invalid_symbol)
            return self._symbols

    @property
    def coins(self):
        """Coins not formatted names for all currencies
        (in lowercase letters) used internally by urls.

        Returns (list):
            All currency coins names provided by coinmarketcap.
        """
        res = self._coins
        if res:
            return res
        else:
            self._coins = []
            for coin_or_coins in list(self.correspondences.values()):
                if isinstance(coin_or_coins, list):
                    for coin in coin_or_coins:
                        self._coins.append(coin)
                else:
                    self._coins.append(coin_or_coins)
            for invalid_coin in INVALID_COINS:
                try:
                    self._coins.remove(invalid_coin)
                except ValueError:
                    pass
            self._coins = sorted(self._coins)
            return self._coins

    @property
    def total_currencies(self):
        res = self._total_currencies
        if res:
            return res
        else:
            self._total_currencies = self.ticker()[-1]["rank"]
            return self._total_currencies

    @property
    def currencies_to_convert(self):
        res = self._currencies_to_convert
        if res:
            return res
        else:
            self._currencies_to_convert = self.__currencies_to_convert()
            return self._currencies_to_convert

    cpdef __currencies_to_convert(self):
        """Internal function for get currencies from and to convert
            values in convert() method. Don't use this, but cached
            :attr:`~pymarketcap.core.Pymarketcap.currencies_to_convert`
            instance attribute instead.

        Returns (list):
            All currencies that could be passed to
            :meth:`~pymarketcap.core.Pymarketcap.convert`.
        """
        res = self._get(b"https://coinmarketcap.com")
        currencies = re_findall(r'data-([a-z]+)="\d', res[-10000:-2000])
        response = [currency.upper() for currency in currencies]
        response.extend([str(currency["symbol"]) for currency in self.ticker()])
        return sorted(response)

    @property
    def exchange_names(self):
        """Get all exchange formatted names provided by coinmarketcap."""
        res = self._exchange_names
        if res:
            return res
        else:
            self._exchange_names = sorted(list(self.__exchange_names_slugs().keys()))
            return self._exchange_names

    cpdef __exchange_names_slugs(self):
        """Internal function for get all exchange names
            available currently in coinmarketcap. Check ``exchange_names``
            instance attribute for the cached method counterpart.

        Returns (list):
            All exchanges names formatted in coinmarketcap.
        """
        res = self._get(b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json")
        return {exc["name"]: exc["slug"] for exc in loads(res)}

    @property
    def exchange_slugs(self):
        """Get all exchange raw names provided by coinmarketcap."""
        res = self._exchange_slugs
        if res:
            return res
        else:
            self._exchange_slugs = sorted(list(self.__exchange_names_slugs().values()))
            return self._exchange_slugs

    @property
    def total_exchanges(self):
        res = self._total_exchanges
        if res:
            return res
        else:
            self._total_exchanges = len(self.exchanges())
            return self._total_exchanges

    @property
    def converter_cache(self):
        res = self._converter_cache
        if res:
            return res
        else:
            self._converter_cache = [self.currency_exchange_rates, time()]
            return self._converter_cache

    cdef _get(self, char *url):
        """Internal function to make and HTTP GET request
        using the curl Cython bridge to C library or urllib
        standard library, depending on the installation."""
        cdef int status
        req = get_to_memory(<char *>url, self.timeout, <bint>self.debug)
        status = req.status_code
        if status == 200:
            return req.text.decode()
        else:
            msg = "Status code -> %d | Url -> %s" % (status, url.decode())
            if status in http_error_numbers:
                raise http_errors_map[str(status)](msg)
            else:
                print("DEBUG: ")
                print(req.text)
                print(req.url)
                raise CoinmarketcapHTTPError(msg)

    # ====================================================================

                        #######   PUBLIC API   #######

    cpdef stats(self, convert="USD"):
        """ Get global cryptocurrencies statistics.

        Args:
            convert (str, optional): return 24h volume, and
                market cap in terms of another currency.
                See ticker_badges property to get valid values.
                As default ``"USD"``.

        Returns (dict):
            Global markets statistics.
        """
        return loads(self._get(
            b"https://api.coinmarketcap.com/v1/global/?convert=%s" % convert.encode()
        ))

    @property
    def ticker_badges(self):
        """Badges in wich you can convert prices in ``ticker()`` method."""
        return

    cpdef ticker(self, currency=None, limit=0, start=0, convert="USD"):
        """Get currencies with other aditional data.

        Args:
            currency (str, optional): Specify a currency to return data,
                that can be a symbol or coin slug (see ``symbols`` and ``coins``
                properties). In this case the method returns a dict, otherwise
                returns a list. If you dont specify a currency,
                returns data for all in coinmarketcap. As default, ``None``.
            limit (int, optional): Limit amount of coins on response.
                If ``limit == 0``, returns all coins in coinmarketcap.
                Only works if ``currency == None``. As default ``0``.
            start (int, optional): Rank of first currency to retrieve.
                The count starts at 0 for the first currency ranked.
                Only works if ``currency == None``. As default ``0``.
            convert (str, optional): Allows to convert prices, 24h volumes
                and market capitalizations in terms of one of badges
                returned by ``ticker_badges`` property. As default, ``"USD"``.

        Returns (dict/list):
            The type depends if currency param is provided or not.
        """
        cdef short i, len_i
        if not currency:
            url =  "https://api.coinmarketcap.com/v1/ticker/?limit=%d" % limit
            url += "&start=%d" % start
            url += "&convert=%s" % convert
            res = self._get(url.encode())
            response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))
            len_i = len(response)
            for i in range(len_i):
                response[i]["symbol"] = str(response[i]["symbol"])
        else:
            if self._is_symbol(currency):
                currency = self.correspondences[currency]
            url = "https://api.coinmarketcap.com/v1/ticker/%s" % currency
            url += "?convert=%s" % convert
            res = self._get(url.encode())
            response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))[0]
            response["symbol"] = str(response["symbol"])
        return response


    # ====================================================================

                       #######    WEB SCRAPER    #######

    @property
    def currency_exchange_rates(self):
        """Currency exchange rates against $ for all currencies (fiat + crypto).

        Returns (dict):
            All currencies rates used internally by coinmarketcap to calculate
            the prices shown.
        """
        res = self._get(b"https://coinmarketcap.com")
        rates = re_findall(r'data-([a-z]+)="(\d+\.*[\d|e|-]*)"', res[-10000:-2000])
        response = {currency.upper(): float(rate) for currency, rate in rates}
        for currency in self.ticker():
            try:
                response[currency["symbol"]] = float(currency["price_usd"])
            except TypeError:
                continue
        return response

    cpdef convert(self, value, unicode currency_in, unicode currency_out):
        """Convert prices between currencies. Provide a value, the currency
        of the value and the currency to convert it and get the value in
        currency converted rate. For see all available currencies to convert
        see ``currencies_to_convert`` property.

        Args:
            value (int/float): Value to convert betweeen two currencies.
            currency_in (str): Currency in which is expressed the value passed.
            currency_out (str): Currency to convert.

        Returns (float):
            Value expressed in currency_out parameter provided.
        """
        if time() - self.converter_cache[1] > 600:
            self.converter_cache[0] = self.currency_exchange_rates
        try:
            if currency_in == "USD":
                return value / self.converter_cache[0][currency_out]
            elif currency_out == "USD":
                return value * self.converter_cache[0][currency_in]
            else:
                rates = self.converter_cache[0]
                return value * rates[currency_in] / rates[currency_out]
        except KeyError:
            msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."
            for param in [currency_in, currency_out]:
                if param not in self.currencies_to_convert:
                    raise ValueError(msg % param)
            raise NotImplementedError

    cpdef currency(self, unicode name, convert="USD"):
        """Get currency metadata like total markets capitalization,
        websites, source code link, if mineable...

        Args:
            currency (str): Currency to get metadata.
            convert (str, optional): Currency to convert response
                fields ``total_markets_cap``, ``total_markets_volume_24h``
                and ``price`` between USD and BTC. As default ``"USD"``.

        Returns (dict):
            Aditional general metadata not supported by other methods.
        """
        response = {}
        if self._is_symbol(name):
            response["symbol"] = name
            name = self.correspondences[name]
            response["slug"] = name
        else:
            response["slug"] = name
            for symbol, slug in self.correspondences.items():
                if slug == name:
                    response["symbol"] = symbol
                    break
        convert = convert.lower()

        try:
            res = self._get(
                b"https://coinmarketcap.com/currencies/%s/" % name.encode()
            )[20000:]
        except CoinmarketcapHTTPError404:
            if name not in self.coins:
                raise ValueError("%s is not a valid currency name. See 'symbols' or 'coins'" % name \
                                 + " properties for get all valid currencies.")
            else:
                raise NotImplementedError

        response.update(processer.currency(res, convert))
        return response

    cpdef markets(self, unicode name, convert="USD"):
        """Get available coinmarketcap markets data.
        It needs a currency as argument.

        Args:
            currency (str): Currency to get market data.
            convert (str, optional): Currency to convert response
                fields ``volume_24h`` and ``price`` between USD
                and BTC. As default ``"USD"``.

        Returns (list):
            Markets on wich provided currency is currently tradeable.
        """
        response = {}
        if self._is_symbol(name):
            response["symbol"] = name
            name = self.correspondences[name]
            response["slug"] = name
        else:
            response["slug"] = name
            for symbol, slug in self.correspondences.items():
                if slug == name:
                    response["symbol"] = symbol
                    break
        convert = convert.lower()

        try:
            res = self._get(
                b"https://coinmarketcap.com/currencies/%s/" % name.encode()
            )[20000:]
        except CoinmarketcapHTTPError404:
            if name not in self.coins:
                raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
                                 + " or 'coins' properties for get all valid currencies.")
            else:
                raise NotImplementedError

        response["markets"] = processer.markets(res, convert)
        return response

    cpdef ranks(self):
        """Returns gainers and losers for 1 hour, 24 hours and 7 days.

        Returns (dict):
            A dictionary with 2 keys (gainers and losers) whose values
            are the periods ``"1h"``, ``"24h"`` and ``"7d"``.
        """
        res = self._get(b"https://coinmarketcap.com/gainers-losers/")

        return processer.ranks(res)

    def historical(self, unicode name,
                   start=datetime(2008, 8, 18),
                   end=datetime.now(),
                   revert=False):
        """Get historical data for a currency.

        Args:
            name (str): Currency to scrap historical data.
            start (date, optional): Time to start scraping
                periods as datetime.datetime type.
                As default ``datetime(2008, 8, 18)``.
            end (date, optional): Time to end scraping periods
                as datetime.datetime type. As default ``datetime.now()``.
            revert (bool, optional): If ``False``, return first date
                first, in chronological order, otherwise returns
                reversed list of periods. As default ``False``.

        Returns (list):
            Historical dayly OHLC for a currency.
        """
        response = {}

        if self._is_symbol(name):
            response["symbol"] = name
            response["slug"] = self.correspondences[name]
            name = response["slug"]
        else:
            response["slug"] = name
            for symbol, slug in self.correspondences.items():
                if slug == name:
                    response["symbol"] = symbol
                    break

        url = "https://coinmarketcap.com/currencies/%s/historical-data/" % name
        _start = "%d" % start.year + "%02d" % start.month + "%02d" % start.day
        _end = "%d" % end.year + "%02d" % end.month + "%02d" % end.day
        url += "?start=%s" % _start + "&" + "end=%s" % _end

        try:
            res = self._get(url.encode())[50000:]
        except CoinmarketcapHTTPError404:
            if name not in self.coins:
                raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
                                 + " or 'coins' properties for get all valid currencies.")
            else:
                raise NotImplementedError

        response["history"] = processer.historical(res, start, end, revert)
        return response

    def recently(self, convert="USD"):
        """Get recently added currencies along with other metadata.

        Args:
            convert (str, optional): Convert market_caps, prices,
                volumes and percent_changes between USD and BTC.
                As default ``"USD"``.

        Returns (list):
            Recently added currencies data.
        """
        convert = convert.lower()
        url = b"https://coinmarketcap.com/new/"
        res = self._get(url)

        return list(processer.recently(res, convert))

    cpdef exchange(self, unicode name, convert="USD"):
        """Obtain data from a exchange passed as argument. See ``exchanges_slugs``
        property for obtain all posibles values.

        Args:
            name (str): Exchange to retrieve data. Check ``exchange_slugs``
                instance attribute for get all posible values passed
                in this parameter.
            convert (str, optional): Convert prices and 24h volumes in
                return between USD and BTC. As default ``"USD"``.

        Returns (dict):
            Data from a exchange. Keys: ``"name"``, ``"website"``,
            ``"volume"`` (total), ``"social"`` and ``"markets"``.
        """
        url = "https://coinmarketcap.com/exchanges/%s/" % name
        try:
            res = self._get(url.encode())[20000:]
        except CoinmarketcapHTTPError404:
            if name not in self.exchange_slugs:
                raise ValueError("%s is not a valid exchange name. See exchange_slugs" % name \
                                 + " property for get all valid exchanges.")
            else:
                raise NotImplementedError
        else:
            response = {"slug": name}
        convert = convert.lower()

        response.update(processer.exchange(res, convert))
        return response

    cpdef exchanges(self, convert="USD"):
        """Get all the exchanges in coinmarketcap ranked by volumes
        along with other metadata.

        Args:
            convert (str, optional): Convert volumes and prices
                between USD and BTC. As default ``"USD"``.

        Returns (list):
            Exchanges with markets and other data included.
        """
        cdef bytes url
        url = b"https://coinmarketcap.com/exchanges/volume/24-hour/all/"
        res = self._get(url)[45000:]
        convert = convert.lower()
        return processer.exchanges(res, convert)

    cpdef tokens(self, convert="USD"):
        """Get data from platforms tokens

        Args:
            convert (str, optional): Convert ``"market_cap"``,
                ``"price"`` and ``"volume_24h"`` values between
                USD and BTC. As default ``"USD"``.

        Returns (list):
            Platforms tokens data.
        """
        url = b"https://coinmarketcap.com/tokens/views/all/"
        res = self._get(url)[40000:]
        convert = convert.lower()

        return processer.tokens(res, convert)

    # ====================================================================

                       #######   INTERNAL API   #######

    cpdef _currency(self, unicode name, start=None, end=None):
        """Get graphs data of a currency.

        Args:
            currency (str): Currency to retrieve graphs data.
            start (datetime, optional): Time to start retrieving
                graphs data in datetime. As default ``None``.
            end (datetime, optional): Time to end retrieving
                graphs data in datetime. As default ``None``.

        Returns (dict):
            Dict info with next keys: ``"market_cap_by_available_supply"``,
            ``"price_btc"``, ``"price_usd"``, ``"volume_usd":``
            and ``"price_platform"``.
            For each value, a list of lists where each one
            has two values [<datetime>, <value>]
        """
        if self._is_symbol(name):
            name = self.correspondences[name]

        url = b"https://graphs2.coinmarketcap.com/currencies/%s/" % name.encode()
        res = loads(self._get(url))

        return processer.graphs(res, start, end)

    cpdef _global_cap(self, bitcoin=True, start=None, end=None):
        """Get global market capitalization graphs, including
        or excluding Bitcoin.

        Args:
            bitcoin (bool, optional): Indicates if Bitcoin will
                be includedin global market capitalization graph.
                As default ``True``.
            start (int, optional): Time to start retrieving
                graphs data in datetime. As default ``None``.
            end (optional, datetime): Time to start retrieving
                graphs data in datetime. As default ``None``.

        Returns (dict):
            Whose values are lists of lists with timestamp and values,
            a data structure with the keys: ``"volume_usd"`` and
            ``"market_cap_by_available_supply"``.
        """
        if bitcoin:
            url = b"https://graphs2.coinmarketcap.com/global/marketcap-total/"
        else:
            url = b"https://graphs2.coinmarketcap.com/global/marketcap-altcoin/"

        res = loads(self._get(url))

        return processer.graphs(res, start, end)

    cpdef _dominance(self, start=None, end=None):
        """Get currencies dominance percentage graph

        Args:
            start (int, optional): Time to start retrieving
                graphs data in datetime. As default ``None``.
            end (optional, datetime): Time to start retrieving
                graphs data in datetime. As default ``None``.

        Returns (dict):
            Altcoins and dominance percentage values with timestamps.
        """
        url = b"https://graphs2.coinmarketcap.com/global/dominance/"

        res = loads(self._get(url))

        return processer.graphs(res, start, end)

    cpdef download_logo(self, unicode name, size=64, filename=None):
        """Download a currency image logo providing their size.

        Args:
            currency (str): Currency name or symbol to download.
            size (int, optional): Size in pixels. Valid sizes are:
                ``[16, 32, 64, 128, 200]``. As default ``128``.
            filename (str, optional): Filename for store the logo.
                Must be in ``.png`` extension. As default ``None``.

        Returns (str):
            Filename of downloaded file if all was correct.
        """
        if self._is_symbol(name):
            try:
                _name = self.ids_correspondences[name]
            except KeyError:
                if name not in list(self.ids_correspondences.keys()):
                    raise ValueError(
                        "The currency %s is not valid. See 'symbols' instance attribute." % name
                    )
        else:
            _name = name

        url_schema = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png"
        url = url_schema % (size, size, _name)
        if not filename:
            filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
        try:
            res = urlretrieve(url, filename)
        except HTTPError as e:
            if e.code == 403:
                valid_sizes = [16, 32, 64, 128, 200]
                if size in valid_sizes:
                    raise ValueError(
                        ("Seems that %s currency doesn't allows to be downloaded with " \
                        + "size %dx%d. Try another size.") % (name, size, size)
                    )
                else:
                    raise ValueError("%dx%d is not a valid size." % (size, size))
            raise e
        else:
            return filename



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""``exchange()`` shared method test module."""

from pymarketcap.tests import type_test

def assert_types(res):
    """Type assertions for ``exchange()`` methods attribute."""
    map_types = {
        "currency":       str,
        "pair":           str,
        "volume_24h":     (float, type(None)),
        "price":          float,
        "percent_volume": float,
        "updated":        bool,
        "name":           str,
        "slug":           str
    }
    assert isinstance(res, dict)

    assert isinstance(res["name"], str)
    assert isinstance(res["web"], (str, type(None)))
    assert isinstance(res["volume"], (float, type(None)))
    assert isinstance(res["social"], dict)

    for key, fields in res["social"].items():
        assert isinstance(key, str)
        for field, data in fields.items():
            assert isinstance(field, str)
            assert isinstance(data, (str, type(None)))

    for market in res["markets"]:
        assert isinstance(market, dict)
        for key, value in market.items():
            type_test(map_types, key, value)

def assert_consistence(res):
    assert res["name"] # len(res["name"]) > 0
    assert res["name"] != None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import choice, uniform
from operator import lt, gt, eq

import pytest

from pymarketcap import Pymarketcap
pym = Pymarketcap()

currencies_to_convert = pym.currencies_to_convert
currencies_to_convert.remove("USD")

def test_USD_to_USD():
    value = uniform(1, 1000)
    res = pym.convert(value, "USD", "USD")

    # Test types
    assert isinstance(res, float)

    # Test consistence
    assert res == value

def test_USD_to_x():
    value = uniform(1, 1000)
    symbol = choice(currencies_to_convert)
    print("(Symbol: %s)" % symbol, end=" ")
    res = pym.convert(value, "USD", symbol)

    # Test types
    assert isinstance(res, float)

    # Test consistence
    rates = pym.currency_exchange_rates
    for op in (lt, gt):
        if not op(rates[symbol], rates["USD"]):  # Inverted
            assert op(res, value)

def test_x_to_USD():
    value = uniform(1, 1000)
    symbol = choice(currencies_to_convert)
    print("(Symbol: %s)" % symbol, end=" ")
    res = pym.convert(value, symbol, "USD")

    # Test types
    assert isinstance(res, float)

    # Test consistence
    rates = pym.currency_exchange_rates
    for op in (lt, gt, eq):
        if op(rates[symbol], rates["USD"]):  # Multiplied
            assert op(res, value)

def test_x_to_y():
    value = uniform(1, 1000)
    symbol_1 = choice(currencies_to_convert)
    symbol_2 = choice(currencies_to_convert)
    print("(Symbol 1: %s | Symbol 2: %s)" \
          % (symbol_1, symbol_2), end=" ")
    res = pym.convert(value, symbol_1, symbol_2)

    # Test types
    assert isinstance(res, float)

    # Test consistence
    rates = pym.currency_exchange_rates
    for op in (lt, gt, eq):
        if op(rates[symbol_1], rates[symbol_2]):  # Multiplied
            assert op(res, value)

def test_invalid_symbol():
    symbol = "OUDBASDFADB"
    with pytest.raises(ValueError) as excinfo:
        pym.convert(0, symbol, "USD")
    assert "Invalid currency: '%s'" % symbol in str(excinfo.value)

def test_zero():
    symbol_1 = choice(currencies_to_convert)
    symbol_2 = choice(currencies_to_convert)
    print("(Symbol 1: %s | Symbol 2: %s)" \
          % (symbol_1, symbol_2), end=" ")
    res = pym.convert(0, symbol_1, symbol_2)
    assert res == 0



# -*- coding: utf-8 -*-

"""Constants used by Pymarketcap interfaces."""

import logging

# Parameters
DEFAULT_TIMEOUT = 15

# Data
EXCEPTIONAL_COIN_SLUGS = {
    "42": "42-coin",
    "808": "808coin",
    "611": "sixeleven",
    "300": "300-token",
    "888": "octocoin",
    "$$$": "money",
    "BTBc": "bitbase",
    "1337": "1337coin",
    "AMMO": "ammo-reloaded",
    "BTW": "bitwhite"
}

EXCEPTIONAL_COIN_SLUGS_KEYS = list(EXCEPTIONAL_COIN_SLUGS.keys())

# Coins retrieved by cache but not found in coinmarketcap
INVALID_COINS = [
    "coindash"
]

# Logging

DEFAULT_FORMAT = "%(asctime)s %(levelname)-8s %(name)s:%(lineno)d: %(message)s"
DEFAULT_FORMATTER = logging.Formatter(DEFAULT_FORMAT)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
from urllib.request import urlopen

def GET(url):
    done = False
    count = 0
    while not done:
        try:
            data = urlopen(url)
        except Exception as e:
            print(e)
            count += 1
            if count >= 3:
                break
        else:
            done = True
            response = data.read()
            data.close()
            return response
    print("Check your internet connection before install pymarketcap.")
    sys.exit(1)


def format_list_as_string(_list, items_each_line, left_indent,
                                  indent_first_line=0):
    """Format a list as string"""
    l_formatted = " "*indent_first_line + " ["
    items_each_line_count = 0
    for item in _list:
        l_formatted += '"%s", ' % item
        items_each_line_count += 1
        if items_each_line_count == items_each_line and item != _list[-1]:
            l_formatted = l_formatted[:-1] + "\n" + " "*left_indent
            items_each_line_count = 0
    return l_formatted[:-2] + "]"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pprint import pprint

import pytest

from pymarketcap import (
    Pymarketcap,
    AsyncPymarketcap,
)

def test_sync_interface(event_loop):
    async def wrapper():
        async with AsyncPymarketcap(debug=True) as apym:
            sync_interface = apym.sync
        assert isinstance(sync_interface, Pymarketcap)
    event_loop.run_until_complete(wrapper())

def test_cached_properties(event_loop):
    async def wrapper():
        async with AsyncPymarketcap(debug=True) as apym:
            properties = [
                "symbols", "coins", "exchange_names", "exchange_slugs"
            ]
            for prop in properties:
                res = eval("apym.%s" % prop)
                assert isinstance(res, list)
                assert len(res) > 0
    event_loop.run_until_complete(wrapper())




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import re

from precompiler import recopiler
from precompiler.utils import format_list_as_string

class Builder:
    """Parser and compiler of the source code of the wrapper
    to create it programatically.

    Args:
        source (str, optional): Path to source file where make changes.
            As default cymarketcap/src/core.pyx
    """
    def __init__(self, source=os.path.join(os.getcwd(), "pymarketcap", "core.pyx")):
        self.source = source
        self.test_sources = {
            "test_ticker": os.path.join(os.getcwd(), "tests", "test_sync_core",
                                        "test_public_api", "test_ticker.py")
        }

    ###   MAIN BUILDER METHODS   ###

    def build(self, func):
        """Run the compiler for build a piece of code.

        Args:
            func (str): Select the func to use for build.

        """
        return eval('self.%s()' % func)

    def read_source(self):
        """Read source file and returns their content."""
        with open(self.source, "r") as f:
            content = f.read()
        return content

    def write_source(self, content):
        """Reaplce content of source file.

        Args:
            content (str): New content to write on source file.
        """
        with open(self.source, "w") as f:
            f.write(content)

    # ==========================================================

    ###   BUILDS METHODS   ###

    def ticker_badges(self):
        # Get badges from coinmarketcap docs
        badges = recopiler.get_badges_list()

        def return_ticker_badges(stream, badges):
            # Get source
            searcher = re.compile(r"\s{4}def ticker_badges.*return\n^$",
            	                  re.MULTILINE | re.DOTALL)
            original_func = searcher.search(stream).group()[:-1]

            badges_formatted = format_list_as_string(badges, 8, 16)

            # Write badges as return of ticker_badges function
            output_func = original_func + badges_formatted
            return stream.replace(original_func, output_func)

        result = return_ticker_badges(self.read_source(), badges)

        # Save result on source file
        self.write_source(result)

    def unskip_tests(self):
        source = self.test_sources["test_ticker"]
        with open(source, "r") as f:
            content = f.readlines()

        result = []
        for line in content:
            if "@pytest.mark.skip" in line and "ticker_badges" in line:
                continue
            result.append(line)

        with open(source, "w") as f:
            f.writelines(result)

    # ===========================================================================

    def unbuild(self):
        """Restaure original state of the code"""
        def return_ticker_badges(stream):
            searcher = re.compile(r"def ticker_badges.+\n.+\n.+return(\s.+\n.+\n.+\n.+)")
            original_func = searcher.search(stream).group(1)
            return stream.replace(original_func, "", 1)

        def restore_curl_import(stream):
            return stream.replace("from pymarketcap.url import get_to_memory",
                                  "from pymarketcap.curl import get_to_memory")

        stream = return_ticker_badges(
            restore_curl_import(
                self.read_source()
            )
        )

        self.write_source(stream)

        def reskip_tests(stream):
            func_name_string = " "*4 + "def test_convert(self):"
            skip_string = " "*4 \
                + '@pytest.mark.skip(reason="ticker_badges property needed but not built yet")\n'
            return stream.replace(func_name_string,
                                  skip_string + func_name_string)

        source = self.test_sources["test_ticker"]
        with open(source, "r") as f:
            stream = f.read()

        with open(source, "w") as f:
            f.write(reskip_tests(stream))

        return True




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap.tests import type_test
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def assert_types(data):
    map_types = {
        "name":           str,
        "symbol":         str,
        "volume_24h":     float,
        "price":          float,
        "percent_change": float,
    }

    assert isinstance(data, dict)
    for rank, rdata in data.items():
        assert isinstance(rank, str)
        assert isinstance(rdata, dict)
        for period, pdata in rdata.items():
            assert isinstance(period, str)
            assert isinstance(pdata, list)
            for currency in pdata:
                assert isinstance(currency, dict)
                for key, value in currency.items():
                    type_test(map_types, key, value)

def test_types():
    assert_types(pym.ranks())

def assert_currencies_order(data):
    # Assert if currencies are in order
    for rank, rdata in data.items():
        for period, pdata in rdata.items():
            for i, currency in enumerate(pdata):
                if i < len(pdata)-1:
                    assert pdata[i+1]["percent_change"] >= currency["percent_change"]

def test_consistence():
    assert_currencies_order(pym.ranks())



# -*- coding: utf-8 -*-

"""Pymarketcap errors module."""

class CoinmarketcapError(Exception):
    """Coinmarketcap base classs errors."""
    pass

                    # HTTP Errors #

class CoinmarketcapHTTPError(CoinmarketcapError):
    """Exception for catch HTTPErrors."""
    pass

class CoinmarketcapHTTPError404(CoinmarketcapError):
    """Exception for catch 404 HTTP error codes."""
    pass

class CoinmarketcapTooManyRequestsError(CoinmarketcapHTTPError):
    """Exception for catch 429 HTTP error codes."""
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import randint
from datetime import datetime, timedelta

from pymarketcap.tests.graphs import assert_types
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def test_types():
    res = pym.graphs.global_cap()
    assert_types(res)

def test_consistence():
    res = pym.graphs.global_cap()
    assert len(res) == 2

    fields = list(res.keys())
    for f in ["market_cap_by_available_supply", "volume_usd"]:
        assert f in fields

def test_start_end():
    today = datetime.now()
    days_ago = today - timedelta(days=randint(1, 15))

    res = pym.graphs.global_cap(start=days_ago)
    assert res["market_cap_by_available_supply"][-1][0].day == today.day
    assert res["market_cap_by_available_supply"][0][0].day in \
        [
         days_ago.day,
         (days_ago+timedelta(days=1)).day,
         (days_ago-timedelta(days=1)).day
        ]
